/**
 * Copyright (c) 2017-2024, Intel Corporation
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of Intel Corporation nor the names of its contributors
 *      may be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#pragma once

#include "sgx_error.h"
#include "se_memcpy.h"

#define UNUSED(val) (void)(val)

#define CHECK_NULL_BREAK(p) if(!p){ break;}
#define CHECK_SGX_ERROR_BREAK(sgx_ret) if(sgx_ret != SGX_SUCCESS){ break;}

#ifndef  BUILD_QAE
#include "se_trace.h"
#include "sgx_ql_lib_common.h"
#include "sgx_report.h"

/**
 * Return QAE's target info which can be used to target an SGX REPORT to the QAE.
 * This is used so that the QAE can verify the integrity of a Verification Result JWT generated by the QVE.
 *
 * @param p_target_info[OUT] - Pointer to the buffer that contains the QAE target information.
 *
 * @return Status code of the operation. SGX_QL_SUCCESS or failure as defined in sgx_ql_lib_common.h
 **/
quote3_error_t tee_qae_get_target_info(sgx_target_info_t *p_target_info);

#else
#include <stdarg.h>
#include <stdio.h>
#include <string.h>

#define SE_TRACE(debug_level, fmt, ...)

#define se_trace(debug_level, fmt, ...)

#endif
