/*
 * Copyright (C) 2011-2021 Intel Corporation. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of Intel Corporation nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


#ifndef SE_DETECT_H_
#define SE_DETECT_H_

#include "se_types.h"
#include "sgx_attributes.h"

#define SE_LEAF 0x12
#define CPUID_FEATURE_FLAGS 7
#define SE_FEATURE_SHIFT 2
#define SE1_SHIFT 0
// ECX[9]
#define PKRU_SHIFT 9

#define AMX_TILECFG_SHIFT 17
#define AMX_TILEDATA_SHIFT 18

#define EDECCSSA_SHIFT 11

// CPUID function 1 
// ECX[26] enums general support for XSAVE
// ECX[27] enums XSAVE is enabled or not
#define XSAVE_SHIFT 26
#define OSXSAVE_SHIFT 27

// CPUID function 0DH, sub-function 1
// EAX[1] enums support for compaction extensions to XSAVE
#define XSAVEC_SHIFT 1

// CPUID function 07H, sub-function 0
// ECX[3] enums whether protection key feature is supported for user mode pages.
// ECX[4] enums whether the OS supports use of the PKRU or not
#define PKU_SHIFT 3
#define PKE_SHIFT 4

bool is_se_supported();

bool is_edeccssa_supported();

bool try_read_xcr0(uint64_t *value);
//bool is_se_debug_supported();
bool get_plat_cap_by_cpuid(sgx_misc_attribute_t *se_misc_attr);

#endif
